<?php
// +----------------------------------------------------------------------
// | 后台基础控制器
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\common\controller;

use app\common\controller\Emptys;
use think\facade\Request;
use app\admin\model\AuthMenu as Menu;
use app\admin\model\AuthAdmin as Admin;
use app\admin\model\AuthRole as Role;
use app\admin\model\AuthAdminAction as ActionLog;
use think\facade\Session;
use think\facade\Cookie;

class _Admin extends Emptys
{
    // 管理员登录实例
    protected $admin = [];
    // 菜单实例
    protected $menu  = null;
    // 默认的公共控制器方法
    protected $public_menu = [
        'login/index',
        'login/handle',
        'login/out',
    ];

    // 后台中间件绑定
    protected $middleware = [
        // 因为中间件带分类，所以我们在绑定的时候也得加上命名空间
        // 公共验证
        'app\http\middleware\admin\Auth'
    ];
    // 代码管理架构
    protected function _autoload($_this) {
        $DS = DIRECTORY_SEPARATOR;
        // 获得控制器名称
        $controller = Request::instance()->controller();
        // 获得操作名称
        $action     = Request::instance()->action();
        // 操作方法文件的绝对地址 
        $url = APP_PATH.'admin'.$DS.'controller'.$DS.$controller.$DS.$action.'.php';
        // 操作方法不存在文件不存在调用基础类404重定向
        if (!file_exists($url)) {
            return $this->__empty();
        }
        // 构造出命名空间
        $url = "\app\admin\controller\\$controller\\$action";
        // 否则加载该文件
        $obj = new $url();
        $numargs = Request::instance()->param();
        // 统一加载run方法
        echo $obj->run($numargs, $_this);
    }

    // 权限验证
    protected function initialize()
    {
        // 先执行父类构造函数
        parent::initialize();
        // 启动模板布局
        $this->view->engine->layout(true);

        // 获得控制器名称
        $controller = Request::instance()->controller();
        // 获得操作名称
        $action     = Request::instance()->action();
        $url        = strtolower($controller.'/'.$action);

        # 七天登录检测
        $this->login_auto();
        
        if (empty($this->admin)) {
            $status = false;
            # 是公共方法则不限制进入
            foreach ($this->public_menu as $v) {
                if ($url == $v) {
                    $status = true;
                    break;
                }
            }
            # 只有未登录的情况下才验证公共方法
            if (!$status) $this->redirect('login/index');
        }

        # 获取列表
        $this->get_menu();

        # 验证访问权限
		$this->vif_menu($url);

        # 将控制器实例返回到模板，方便进行验证
		$this->assign('auth', $this);
    }

    // 验证访问权限
	private function vif_menu($url){
		$status = false;
		# 验证菜单权限
		foreach ($this->public_menu as $v) {
			if ($v == $url) {
				$status = true;
                break;
			}
        }
        
        // 访问不通过，直接调用404
		if (!$status) {
            $this->__empty();
            // 这里没办法只能exit，不然会再次触发框架的路由不存在定向
            exit;
        }
    }

    //七天自动登录
    private function login_auto() {
        $admin = Session::get('admin');

        # 获得七天登录加密串
        $pod = Cookie::get('login_auto');
        # 判断是否已经登录，未登录做七天登录验证
		if (empty($admin) && $pod) {
            $array = explode('|', $pod);
            # 获得用户名
            $user  = \org\Encryption::decrypt($array[0]);
            # 获得加密签名
            $sha1  = !empty($array[1]) ? $array[1] : '';
            # 查询管理员数据
            $info = Admin::alias('A')
                ->join('auth_role B', 'A.role = B.id', 'LEFT')
                ->field('A.*, B.menu_list, B.status as role_status')
                ->where('A.name', $user)
                ->whereOr('A.phone', $user)
                ->find();

            if ($info) {
                # 生成签名
                $sign   = sha1($info['auto_rand'].$info['auto_time'].$info['auto_ip']);
                # 允许登录状态
                $status = true;
                # 七天超时
                if ($info['auto_time'] < time()) { $status = false; }
                # ip不一致
                if ($info['auto_ip'] != Request::instance()->ip()) { $status = false; }
                # 签名不一致
                if (empty($sha1) || $sign != $sha1) { $status = false; }
                # 不成功则清除七天绑定
                if (!$status) {
                    Cookie::set('login_auto', '');
                    Admin::where('id', $info['id'])->update([
                        'auto_time' => '',
                        'auto_rand' => '',
                        'auto_ip'   => '',
                    ]);
                } else {
                    $admin = $info;
                }
            }
        }
        
        # 登录信息绑定
        $this->admin = $admin;
        $this->assign('admin_data', $admin);
    }

    // 单独验证菜单权限
	public function vif($controller, $action){
		$controller = strtolower( $controller );
		$action     = strtolower( $action );
        $status     = false;
        $url        = $controller.'/'.$action;

		# 验证
		foreach ($this->public_menu as $v) {
			if ($v == $url) {
				$status = true;
                break;
			}
		}
		return $status;
	}

    // 获取对应的权限菜单
    protected function get_menu(){
        if ($this->menu != null) return $this->menu;
        $field = 'id, pid, name, display, controller, action, icon';

        $where = [
            'status' => 1
        ];
        
        $list2 = [];
        # 超级管理员
        if ($this->admin['role'] == 0) {
            $list1 = Menu::where($where)->field($field)->order('sort DESC')->select();
        }else{
            # 取出角色下的权限菜单
            $role_menu = Role::where('id', $this->admin['role'])->value('menu_list');
            $list1 = Menu::where($where)->whereIn('id', $role_menu)->field($field)->order('sort DESC')->select();
            # 取出管理员的独立权限
            $admin_menu = \think\Db::name('auth_admin_menu')->where('admin', $this->admin['id'])->value('menu_list');
            if ($admin_menu) {
                $list2 = Menu::where($where)->whereIn('id', $admin_menu)->whereNotIn('id', $role_menu)->field($field)->order('sort DESC')->select();
            }
            # 不是超级管理员还得单独拿公共权限
            $where['public'] = 1;
            $list3 = Menu::where($where)->field($field)->select();
            foreach ($list3 as $v) {
                $this->menu[] = $v;
                $this->public_menu[] = $v['controller'].'/'.$v['action'];
            }
        }
        # 植入权限列表
        foreach ($list1 as $v) {
            $this->menu[] = $v;
            $this->public_menu[] = $v['controller'].'/'.$v['action'];
        }
        foreach ($list2 as $v) {
            $this->menu[] = $v;
            $this->public_menu[] = $v['controller'].'/'.$v['action'];
        }
        return $this->menu;
    }

    // 用于后台递归生成权限菜单
    protected static function menu_make($cate='', $name = 'children', $pid = 0) {
        $arr = [];
        foreach ($cate as $v) {
            if ($v['pid'] == $pid) {
                if ($v['display'] == 1) {
                    $v[$name] = self::menu_make($cate, $name, $v['id']);
                    $v['num'] = count($v[$name]);
                    $arr[] = $v;
                } 
            }
        }
        return $arr;
    }

    // 用于后台输出JSON，并写入日志
    protected function _json($code, $msg, $list=[], $echo=true) {
        $controller = Request::instance()->controller();
        $action     = Request::instance()->action();
        $url        = strtolower($controller.'/'.$action);
        $ip         = Request::instance()->ip();
        $param      = json_encode(Request::instance()->param(), JSON_UNESCAPED_UNICODE);

        $admin_id = $this->admin['id'];
        if (!$admin_id) {
            $admin = Session::get('admin');
            if ($admin) {
               $admin_id = $admin['id']; 
            }
        }

        // 管理员操作日志-分表存储-方式为hash管理员ID
        if ($admin_id) {
            $table = \org\SubTable::init('auth_admin_action')->hash($admin_id);
            \think\Db::name($table)->insert([
                'id' => \org\Snowflake::get_id(),
                'admin' => $admin_id,
                'url' => $url,
                'ip' => $ip,
                'msg' => $msg,
                'param' => $param,
                'create_time' => time(),
            ]);
        }
        
        if ($echo) {
            return json_encode([
                'code' => "$code",
                'msg' => $msg,
                'data' => $list
            ], JSON_UNESCAPED_UNICODE);
        }

    }

    // 用于后台输出LIST菜单给Layui
    protected static function _list($code, $msg, $count, $list=[]) {
        return json_encode([
            'code' => "$code",
            'msg' => $msg,
            'count' => $count,
            'data' => $list
        ], JSON_UNESCAPED_UNICODE);
    }

    // 用于后台判断不能为空，并且可以为0的判断
    protected static function isset_empty($param, $key) {
        if (!isset($param[$key])) return false;
        if (trim($param[$key]) == '') return false;
        return true;
    }

}