<?php
// +----------------------------------------------------------------------
// | 日志操作
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace log;
use think\facade\Config;
use think\facade\Request;

class Log {
    // 换行符
    private static $RN = "\r\n";
    // 内容换行符
    private static $HF = "=========";
    // 日志路径
    private static $log_path;
    // 单例实例
    private static $instance;
    // 日志颜色
    private static $color = [
        'normal'  => '#333', // 正常
        'warning' => '#1ba1e2', // 警告
        'error'   => 'red', // 自定义错误
        'app_error' => 'red', // 系统级的错误
    ];
    
	// 私有化构造函数，可以保证在类外该类不能被实例化
    private function __construct(){}
        
    // 禁止克隆单例类，防止二次new
    private function __clone()
    { 
        die('此对象不允许被克隆');
    }

    /**
     * 用于初始化单例类
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.23
     * @deprecated 暂不弃用
     * @global 无
     * @param string $table
     * @return void
    */
    public static function init($status = true)
    {
        $config = Config::pull('laf');
        if (!self::$instance instanceof self) {
            // 获得单例
            self::$instance = new self();
            // 只有开启才做记录
            if ($status) {
                // 获得日志路径
                self::get_path();
                // 写入启动日志
                self::insert('开始记录', 'normal');
                self::insert('请求地址：'.Request::url(true), 'normal');
                self::insert('请求类型：'.Request::method(true), 'normal');
                self::insert('请求参数：'.json_encode(Request::instance()->param(), JSON_UNESCAPED_UNICODE), 'normal');
                // 开启调试模式
                if ($config['log_debug'] == true) {
                    // 设置错误异常捕捉
                    set_error_handler(array(\log\Log::init(),'error_handler'));
                    set_exception_handler(array(\log\Log::init(),'exception_handler'));
                    # 致命错误捕捉-同时记录结束程序
                    register_shutdown_function(function(){
                        if ($e = error_get_last()) {
                            $error = [];
                            switch($e['type']){
                            case E_ERROR:
                            case E_PARSE:
                            case E_CORE_ERROR:
                            case E_COMPILE_ERROR:
                            case E_USER_ERROR:  
                                ob_end_clean();
                                $error['message'] = $e['message'];
                                $error['file'] = $e['file'];
                                $error['line'] = $e['line'];
                                $html = '';
                                $html .= date('Y-m-d H:i:s', time()).self::$HF;
                                $html .= '发生致命错误了：'.$error['message'].self::$HF;
                                $html .= $error['file'].self::$HF;
                                $html .= '没有捕捉到函数位置'.self::$HF;
                                $html .= $error['line'].self::$HF;
                                $html .= 'app_error'.self::$RN;
                                file_put_contents(self::$log_path, $html, FILE_APPEND);
                                break;
                            }
                        }
                        self::insert('结束记录', 'normal');
                    });
                }
            }
            // 返回单例
            return self::$instance;
        }

        return self::$instance;
    }

    //记录日志
    public static function insert($title, $type='normal') { 
        $config = Config::pull('laf');
        // 不开启调试模式
        if ($config['log_debug'] != true) return false;

        $row = current(debug_backtrace());

        $html = '';
        $html .= date('Y-m-d H:i:s', time()).self::$HF;
        $html .= $title.self::$HF;
        $html .= $row['file'].self::$HF;
        $html .= $row['function'].self::$HF;
        $html .= $row['line'].self::$HF;
        $html .= $type.self::$RN;

        return file_put_contents(self::$log_path, $html, FILE_APPEND);
    }

    // 删除指定目录
    public function file_delete($file) {
        $path = ROOT_PATH.__LOG__.'/'.$file;
        if (is_dir($path)) {
            //扫描一个目录内的所有目录和文件并返回数组
            $dirs = scandir($path);
            foreach ($dirs as $dir) {
                //排除目录中的当前目录(.)和上一级目录(..)
                if ($dir != '.' && $dir != '..') {
                    //如果是目录则递归子目录，继续操作
                    $sonDir = $path.'/'.$dir;
                    if (is_dir($sonDir)) {
                        //递归删除
                        $this->deleteDir($sonDir);
                        //目录内的子目录和文件删除后删除空目录
                        @rmdir($sonDir);
                    } else {
                        //如果是文件直接删除
                        @unlink($sonDir);

                    }
                }
            }
            @rmdir($path);
        }
        return true;
    }

    
    // 删除所有日志
    public function delete() {
        $path = ROOT_PATH.__LOG__.'/';
        //扫描一个目录内的所有目录和文件并返回数组
        $dirs = scandir($path);
        foreach ($dirs as $dir) {
            //排除目录中的当前目录(.)和上一级目录(..)
            if ($dir != '.' && $dir != '..') {
                //如果是目录则递归子目录，继续操作
                $sonDir = $path.'/'.$dir;
                $dirss = scandir($sonDir);
                foreach ($dirss as $file) {
                    //排除目录中的当前目录(.)和上一级目录(..)
                    if ($file != '.' && $file != '..') {
                        //如果是文件直接删除
                        @unlink($sonDir.'/'.$file);
                    }
                }
                //目录内的子目录和文件删除后删除空目录
                @rmdir($sonDir);
            }
        }
        return true;
    }

    // 获取日志文件列表
    public function get_file($file, $page=1, $limit=2) {
        $res = scandir(ROOT_PATH.__LOG__.'/'.$file, 1);
        $list = [];
        $i = 0;
        foreach ($res as $v) {
            if ($v != '.' && $v != '..') {
                $list[$i] = [
                    'title' => $v,
                    'size' => round(filesize(ROOT_PATH.__LOG__.'/'.$file.'/'.$v) / 1024, 2).'kb',
                ];
                $i++;
            }
        }
        $num = count($list);
        $total = ceil($num / $limit);
        $list = $this->ArrayPage($list, $num, $limit, $page);
        return [
            'total' => $num,
            'list' => $list
        ];
    }

    // 获取日志目录列表
    public function get_list($page=1, $limit=2) {
        $res = scandir(ROOT_PATH.__LOG__, 1);
        $list = [];
        $i = 0;
        foreach ($res as $v) {
            if ($v != '.' && $v != '..') {
                $list[$i] = [
                    'title' => $v,
                ];
                $i++;
            }
        }
        $num = count($list);
        $total = ceil($num / $limit);
        $list = $this->ArrayPage($list, $num, $limit, $page);
        return [
            'total' => $num,
            'list' => $list
        ];
    }

    // 数组分页控制
    public function ArrayPage($array, $num, $size, $page){
        if($page==0 && $page==''){
            $page = 1;
        }else{
            $page = $page?$page:1;
        }
        $arr = array();
        for($i = 0; $i < $size; $i++){
            if (empty($array[($page - 1) * $size + $i])) break;
            $arr[] = $array[($page - 1) * $size + $i];
        }
        return $arr;
    }
    
    // 获取日志详情
    public function get_log($log_path) {
        $txt = file_get_contents(ROOT_PATH.__LOG__.'/'.$log_path);
        if (!$txt) return [];
        $array = explode(self::$RN, $txt);
        unset($array[count($array)-1]);

        $list = [];
        foreach ($array as $k=>$v) {
            $arr = explode(self::$HF, $v);
            $list[$k] = [
                'date' => $arr[0],
                'title' => $arr[1],
                'file' => $arr[2],
                'function' => $arr[3],
                'line' => $arr[4],
                'type' => $arr[5],
                'color' => !empty(self::$color[$arr[5]]) ? self::$color[$arr[5]] : '#000',
            ];
        }

        return $list;
    }

    // 判断日志文件是否存在，不存在则创建
    public static function get_path() {
        $config = Config::pull('laf');
        // 不开启调试模式
        if ($config['log_debug'] != true) return false;
            
        $path = ROOT_PATH.__LOG__.'/'.date('Y-m-d').'/';
        if (!file_exists($path)) {
            mkdir($path);
        }
        // 获得模块名称
        $module = Request::instance()->module();
        // 获得控制器名称
        $controller = Request::instance()->controller();
        // 获得操作名称
        $action     = Request::instance()->action();

        self::$log_path = $path.$module.'_'.$controller.'_'.$action.'_'.date('H_i_s').'-'.rand(100, 999).'.log';
    }

    // 警告错误监听
    public function error_handler($type, $message, $file, $line){
        $html = '';
        $html .= date('Y-m-d H:i:s', time()).self::$HF;
        $html .= '发生警告错误了：'.$message.self::$HF;
        $html .= $file.self::$HF;
        $html .= '没有捕捉到函数位置'.self::$HF;
        $html .= $line.self::$HF;
        $html .= 'app_error'.self::$RN;
        file_put_contents(self::$log_path, $html, FILE_APPEND);
        die('发生警告错误了：'.$file.'，Line：'.$line.'，Message：'.$message);
    }
    // 异常错误监听
    public function exception_handler($error){
        $html = '';
        $html .= date('Y-m-d H:i:s', time()).self::$HF;
        $html .= '发生异常错误了：'.$error->getMessage().self::$HF;
        $html .= $error->getFile().self::$HF;
        $html .= '没有捕捉到函数位置'.self::$HF;
        $html .= $error->getLine().self::$HF;
        $html .= 'app_error'.self::$RN;
        file_put_contents(self::$log_path, $html, FILE_APPEND);
        die('发生异常错误了：'.$error->getFile().'，Line：'.$error->getLine().'，Message：'.$error->getMessage());
    }
    
}
